@@ -38,11 +38,11 @@
 #include <QProcess>
 
 #include <KLocalizedString>
-#include <kencodingprober.h>
 
 #include "man2html.h"
 #include <assert.h>
 #include <kfilterdev.h>
+#include <kio_version.h>
 
 using namespace KIO;
 
@@ -112,9 +112,9 @@ bool parseUrl(const QString& _url, QString &title, QString &section)
 
     pos = section.indexOf(')');
     if (pos >= 0) {
-	if (pos < section.length() - 2 && title.isEmpty()) {
-		title = section.mid(pos + 2);
-	}
+        if (pos < section.length() - 2 && title.isEmpty()) {
+                title = section.mid(pos + 2);
+        }
         section = section.left(pos);
     }
 
@@ -150,21 +150,21 @@ void MANProtocol::parseWhatIs( QMap<QString, QString> &i, QTextStream &t, const
     QString l;
     while ( !t.atEnd() )
     {
-	l = t.readLine();
-	int pos = re.indexIn( l );
-	if (pos != -1)
-	{
-	    QString names = l.left(pos);
-	    QString descr = l.mid(pos + re.matchedLength());
-	    while ((pos = names.indexOf(",")) != -1)
-	    {
-		i[names.left(pos++)] = descr;
-		while (names[pos] == ' ')
-		    pos++;
-		names = names.mid(pos);
-	    }
-	    i[names] = descr;
-	}
+        l = t.readLine();
+        int pos = re.indexIn( l );
+        if (pos != -1)
+        {
+            QString names = l.left(pos);
+            QString descr = l.mid(pos + re.matchedLength());
+            while ((pos = names.indexOf(",")) != -1)
+            {
+                i[names.left(pos++)] = descr;
+                while (names[pos] == ' ')
+                    pos++;
+                names = names.mid(pos);
+            }
+            i[names] = descr;
+        }
     }
 }
 
@@ -198,14 +198,14 @@ QMap<QString, QString> MANProtocol::buildIndexMap(const QString &section)
           ++it_dir )
     {
         if ( QFile::exists( *it_dir ) ) {
-    	    QStringList::ConstIterator it_name;
+            QStringList::ConstIterator it_name;
             for ( it_name = names.constBegin();
-	          it_name != names.constEnd();
-	          it_name++ )
+                  it_name != names.constEnd();
+                  it_name++ )
             {
-	        if (addWhatIs(i, (*it_dir) + '/' + (*it_name), mark))
-		    break;
-	    }
+                if (addWhatIs(i, (*it_dir) + '/' + (*it_name), mark))
+                    break;
+            }
             if ( it_name == names.constEnd() ) {
                 QProcess proc;
                 proc.setProgram("whatis");
@@ -221,6 +221,8 @@ QMap<QString, QString> MANProtocol::buildIndexMap(const QString &section)
     return i;
 }
 
+//---------------------------------------------------------------------
+
 QStringList MANProtocol::manDirectories()
 {
     checkManPaths();
@@ -306,8 +308,8 @@ QStringList MANProtocol::findPages(const QString &_section,
     //
     for ( int i=0;i<sect_list.count(); i++)
     {
-            QString it_s=sect_list.at(i);
-	    QString it_real = it_s.toLower();
+        QString it_s=sect_list.at(i);
+        QString it_real = it_s.toLower();
         //
         // Find pages
         //
@@ -340,7 +342,7 @@ QStringList MANProtocol::findPages(const QString &_section,
                 else if (file.startsWith(sman))
                     sect = file.mid(4);
 
-		if (sect.toLower()==it_real) it_real = sect;
+                if (sect.toLower()==it_real) it_real = sect;
 
                 // Only add sect if not already contained, avoid duplicates
                 if (!sect_list.contains(sect) && _section.isEmpty())  {
@@ -529,12 +531,12 @@ void MANProtocol::get(const QUrl& url )
 char *MANProtocol::readManPage(const char *_filename)
 {
     QByteArray filename = _filename;
-    QByteArray array;
+    QByteArray array, dirName;
 
     /* Determine type of man page file by checking its path. Determination by
-     * MIME type with KMimeType doesn't work reliablely. E.g., Solaris 7:
+     * MIME type with KMimeType doesn't work reliably. E.g., Solaris 7:
      * /usr/man/sman7fs/pcfs.7fs -> text/x-csrc : WRONG
-     * If the path name constains the string sman, assume that it's SGML and
+     * If the path name contains the string sman, assume that it's SGML and
      * convert it to roff format (used on Solaris). */
     //QString file_mimetype = KMimeType::findByPath(QString(filename), 0, false)->name();
     if (QString(filename).contains("sman", Qt::CaseInsensitive)) //file_mimetype == "text/html" || )
@@ -560,6 +562,11 @@ char *MANProtocol::readManPage(const char *_filename)
 
       lastdir = filename.left(filename.lastIndexOf('/'));
 
+      // get the last directory name (which might be a language name, to be able to guess the encoding)
+      QDir dir(lastdir);
+      dir.cdUp();
+      dirName = QFile::encodeName(dir.dirName());
+
       if ( !QFile::exists(QFile::decodeName(filename)) )  // if given file does not exist, find with suffix
       {
           qCDebug(KIO_MAN_LOG) << "not existing " << filename;
@@ -569,38 +576,19 @@ char *MANProtocol::readManPage(const char *_filename)
           qCDebug(KIO_MAN_LOG) << "resolved to " << filename;
       }
 
-      QIODevice *fd = KFilterDev::deviceForFile(filename);
+      KFilterDev fd(QFile::encodeName(filename));
 
-      if ( !fd || !fd->open(QIODevice::ReadOnly))
-      {
-         delete fd;
+      if ( !fd.open(QIODevice::ReadOnly))
          return nullptr;
-      }
-      array = fd->readAll();
+
+      array = fd.readAll();
       qCDebug(KIO_MAN_LOG) << "read " << array.size();
-      fd->close();
-      delete fd;
     }
 
     if (array.isEmpty())
       return nullptr;
 
-    // as we do not know in which encoding the man source is, try to automatically
-    // detect it and always return it as UTF-8
-    KEncodingProber encodingProber;
-    encodingProber.feed(array);
-    qCDebug(KIO_MAN_LOG) << "auto-detect encoding for" << filename << "guess=" << encodingProber.encoding()
-                 << "confidence=" << encodingProber.confidence();
-    QString out = QTextCodec::codecForName(encodingProber.encoding())->toUnicode(array);
-    array = out.toUtf8();
-
-    const int len = array.size();
-    char *buf = new char[len + 4];
-    memmove(buf + 1, array.data(), len);
-    buf[0] = buf[len+1] = '\n'; // Start and end with an end of line
-    buf[len+2] = buf[len+3] = '\0'; // Two NUL characters at end
-
-    return buf;
+    return manPageToUtf8(array, dirName);
 }
 
 //---------------------------------------------------------------------
@@ -671,8 +659,15 @@ void MANProtocol::stat( const QUrl& url)
     qCDebug(KIO_MAN_LOG) << "URL " << url.url() << " parsed to title='" << title << "' section=" << section;
 
     UDSEntry entry;
+#if (KIO_VERSION >= QT_VERSION_CHECK(5, 48, 0))
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, title);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+    entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("text/html"));
+#else
     entry.insert(KIO::UDSEntry::UDS_NAME, title);
     entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+    entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("text/html"));
+#endif
 
 #if 0 // not useful, is it?
     QString newUrl = "man:"+title;
@@ -681,8 +676,6 @@ void MANProtocol::stat( const QUrl& url)
     entry.insert(KIO::UDSEntry::UDS_URL, newUrl);
 #endif
 
-    entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("text/html"));
-
     statEntry(entry);
 
     finished();
@@ -748,18 +741,18 @@ QStringList MANProtocol::buildSectionList(const QStringList& dirs) const
     QStringList l;
 
     for (QStringList::ConstIterator it = section_names.begin();
-	    it != section_names.end(); ++it)
+            it != section_names.end(); ++it)
     {
-	    for (QStringList::ConstIterator dir = dirs.begin();
-		    dir != dirs.end(); ++dir)
-	    {
-		QDir d((*dir)+"/man"+(*it));
-		if (d.exists())
-		{
-		    l << *it;
-		    break;
-		}
-	    }
+        for (QStringList::ConstIterator dir = dirs.begin();
+             dir != dirs.end(); ++dir)
+        {
+            QDir d((*dir)+"/man"+(*it));
+            if (d.exists())
+            {
+                l << *it;
+                break;
+            }
+        }
     }
     return l;
 }
@@ -783,9 +776,9 @@ void MANProtocol::showMainIndex()
     const QString sectList = getenv("MANSECT");
     QStringList sections;
     if (sectList.isEmpty())
-    	sections = buildSectionList(manDirectories());
+        sections = buildSectionList(manDirectories());
     else
-	sections = sectList.split( ':');
+        sections = sectList.split(':');
 
     os << "<table>" << endl;
 
@@ -803,8 +796,8 @@ void MANProtocol::showMainIndex()
         accessKeys.insert(accessKey);
 
         os << "<tr><td><a href=\"man:(" << *it << ")\" accesskey=\"" << accessKey
-	<< "\">" << i18n("Section %1", *it)
-	<< "</a></td><td>&nbsp;</td><td> " << sectionName(*it) << "</td></tr>" << endl;
+           << "\">" << i18n("Section %1", *it)
+           << "</a></td><td>&nbsp;</td><td> " << sectionName(*it) << "</td></tr>" << endl;
     }
 
     os << "</table>" << endl;
@@ -1010,7 +1003,7 @@ void MANProtocol::checkManPaths()
         construct_path = true; // need to read config file
     }
 
-    // Constucted man path -- consists of paths from
+    // Constructed man path -- consists of paths from
     //   /etc/man.conf
     //   default dirs
     //   $PATH
@@ -1115,27 +1108,27 @@ int compare_man_index(const void *s1, const void *s2)
     // this is a bit tricky
     if ( m1->manpage_len > m2->manpage_len)
     {
-	i = qstrnicmp( m1->manpage_begin,
-		      m2->manpage_begin,
-		      m2->manpage_len);
-	if (!i)
-	    return 1;
-	return i;
+        i = qstrnicmp(m1->manpage_begin,
+                      m2->manpage_begin,
+                      m2->manpage_len);
+        if (!i)
+            return 1;
+        return i;
     }
 
     if ( m1->manpage_len < m2->manpage_len)
     {
-	i = qstrnicmp( m1->manpage_begin,
-		      m2->manpage_begin,
-		      m1->manpage_len);
-	if (!i)
-	    return -1;
-	return i;
+        i = qstrnicmp(m1->manpage_begin,
+                      m2->manpage_begin,
+                      m1->manpage_len);
+        if (!i)
+            return -1;
+        return i;
     }
 
-    return qstrnicmp( m1->manpage_begin,
-		     m2->manpage_begin,
-		     m1->manpage_len);
+    return qstrnicmp(m1->manpage_begin,
+                     m2->manpage_begin,
+                     m1->manpage_len);
 }
 
 void MANProtocol::showIndex(const QString& section)
@@ -1189,76 +1182,76 @@ void MANProtocol::showIndex(const QString& section)
     QStringList::const_iterator page;
     for (page = pages.constBegin(); page != pages.constEnd(); ++page)
     {
-	// I look for the beginning of the man page name
-	// i.e. "bla/pagename.3.gz" by looking for the last "/"
-	// Then look for the end of the name by searching backwards
-	// for the last ".", not counting zip extensions.
-	// If the len of the name is >0,
-	// store it in the list structure, to be sorted later
+        // I look for the beginning of the man page name
+        // i.e. "bla/pagename.3.gz" by looking for the last "/"
+        // Then look for the end of the name by searching backwards
+        // for the last ".", not counting zip extensions.
+        // If the len of the name is >0,
+        // store it in the list structure, to be sorted later
 
         char *manpage_end;
         struct man_index_t *manindex = new man_index_t;
-	manindex->manpath = strdup((*page).toUtf8());
+        manindex->manpath = strdup((*page).toUtf8());
 
-	manindex->manpage_begin = strrchr(manindex->manpath, '/');
-	if (manindex->manpage_begin)
-	{
-	    manindex->manpage_begin++;
-	    assert(manindex->manpage_begin >= manindex->manpath);
-	}
-	else
-	{
-	    manindex->manpage_begin = manindex->manpath;
-	    assert(manindex->manpage_begin >= manindex->manpath);
-	}
+        manindex->manpage_begin = strrchr(manindex->manpath, '/');
+        if (manindex->manpage_begin)
+        {
+            manindex->manpage_begin++;
+            assert(manindex->manpage_begin >= manindex->manpath);
+        }
+        else
+        {
+            manindex->manpage_begin = manindex->manpath;
+            assert(manindex->manpage_begin >= manindex->manpath);
+        }
 
-	// Skip extension ".section[.gz]"
+        // Skip extension ".section[.gz]"
 
-	char *begin = (char*)(manindex->manpage_begin);
-	int len = strlen( begin );
-	char *end = begin+(len-1);
+        char *begin = (char*)(manindex->manpage_begin);
+        int len = strlen( begin );
+        char *end = begin+(len-1);
 
-	if ( len >= 3 && strcmp( end-2, ".gz" ) == 0 )
-	    end -= 3;
-	else if ( len >= 2 && strcmp( end-1, ".Z" ) == 0 )
-	    end -= 2;
-	else if ( len >= 2 && strcmp( end-1, ".z" ) == 0 )
-	    end -= 2;
-	else if ( len >= 4 && strcmp( end-3, ".bz2" ) == 0 )
-	    end -= 4;
-	else if ( len >= 5 && strcmp( end-4, ".lzma" ) == 0 )
-	    end -= 5;
-	else if ( len >= 3 && strcmp( end-2, ".xz" ) == 0 )
-	    end -= 3;
+        if ( len >= 3 && strcmp( end-2, ".gz" ) == 0 )
+            end -= 3;
+        else if ( len >= 2 && strcmp( end-1, ".Z" ) == 0 )
+            end -= 2;
+        else if ( len >= 2 && strcmp( end-1, ".z" ) == 0 )
+            end -= 2;
+        else if ( len >= 4 && strcmp( end-3, ".bz2" ) == 0 )
+            end -= 4;
+        else if ( len >= 5 && strcmp( end-4, ".lzma" ) == 0 )
+            end -= 5;
+        else if ( len >= 3 && strcmp( end-2, ".xz" ) == 0 )
+            end -= 3;
 
-	while ( end >= begin && *end != '.' )
-	    end--;
+        while ( end >= begin && *end != '.' )
+            end--;
 
-	if ( end < begin )
-	    manpage_end = nullptr;
-	else
-	    manpage_end = end;
+        if ( end < begin )
+            manpage_end = nullptr;
+        else
+            manpage_end = end;
 
-	if (nullptr == manpage_end)
-	{
-	    // no '.' ending ???
-	    // set the pointer past the end of the filename
-	    manindex->manpage_len = (*page).length();
-	    manindex->manpage_len -= (manindex->manpage_begin - manindex->manpath);
-	    assert(manindex->manpage_len >= 0);
-	}
-	else
-	{
-	    manindex->manpage_len = (manpage_end - manindex->manpage_begin);
-	    assert(manindex->manpage_len >= 0);
-	}
+        if (nullptr == manpage_end)
+        {
+            // no '.' ending ???
+            // set the pointer past the end of the filename
+            manindex->manpage_len = (*page).length();
+            manindex->manpage_len -= (manindex->manpage_begin - manindex->manpath);
+            assert(manindex->manpage_len >= 0);
+        }
+        else
+        {
+            manindex->manpage_len = (manpage_end - manindex->manpage_begin);
+            assert(manindex->manpage_len >= 0);
+        }
 
-	if (0 < manindex->manpage_len)
-	{
-	    indexlist[listlen] = manindex;
-	    listlen++;
-	}
-	else delete manindex;
+        if (0 < manindex->manpage_len)
+        {
+            indexlist[listlen] = manindex;
+            listlen++;
+        }
+        else delete manindex;
     }
 
     //
@@ -1277,60 +1270,60 @@ void MANProtocol::showIndex(const QString& section)
     QString indexLine="<div class=\"secidxshort\">\n";
     if (indexlist[0]->manpage_len>0)
     {
-	firstchar=QChar((indexlist[0]->manpage_begin)[0]).toLower();
+        firstchar=QChar((indexlist[0]->manpage_begin)[0]).toLower();
 
-	const QString appendixstr = QString(
-	    " [<a href=\"#%1\" accesskey=\"%2\">%3</a>]\n"
-	).arg(firstchar).arg(firstchar).arg(firstchar);
-	indexLine.append(appendixstr);
+        const QString appendixstr = QString(
+            " [<a href=\"#%1\" accesskey=\"%2\">%3</a>]\n"
+        ).arg(firstchar).arg(firstchar).arg(firstchar);
+        indexLine.append(appendixstr);
     }
     os << "<tr><td class=\"secidxnextletter\"" << " colspan=\"3\">\n  <a name=\""
        << firstchar << "\">" << firstchar <<"</a>\n</td></tr>" << endl;
 
     for (int i=0; i<listlen; i++)
     {
-	struct man_index_t *manindex = indexlist[i];
+        struct man_index_t *manindex = indexlist[i];
 
-	// qstrncmp():
-	// "last_man" has already a \0 string ending, but
-	// "manindex->manpage_begin" has not,
-	// so do compare at most "manindex->manpage_len" of the strings.
-	if (last_index->manpage_len == manindex->manpage_len &&
-	    !qstrncmp(last_index->manpage_begin,
-		      manindex->manpage_begin,
-		      manindex->manpage_len)
-	    )
-	{
-	    continue;
-	}
+        // qstrncmp():
+        // "last_man" has already a \0 string ending, but
+        // "manindex->manpage_begin" has not,
+        // so do compare at most "manindex->manpage_len" of the strings.
+        if (last_index->manpage_len == manindex->manpage_len &&
+            !qstrncmp(last_index->manpage_begin,
+                      manindex->manpage_begin,
+                      manindex->manpage_len)
+            )
+        {
+            continue;
+        }
 
-	tmp=QChar((manindex->manpage_begin)[0]).toLower();
-	if (firstchar != tmp)
-	{
-	    firstchar = tmp;
-	    os << "<tr><td class=\"secidxnextletter\"" << " colspan=\"3\">\n  <a name=\""
-	       << firstchar << "\">" << firstchar << "</a>\n</td></tr>" << endl;
+        tmp=QChar((manindex->manpage_begin)[0]).toLower();
+        if (firstchar != tmp)
+        {
+            firstchar = tmp;
+            os << "<tr><td class=\"secidxnextletter\"" << " colspan=\"3\">\n  <a name=\""
+               << firstchar << "\">" << firstchar << "</a>\n</td></tr>" << endl;
 
-    	    const QString appendixstr = QString(
-    		" [<a href=\"#%1\" accesskey=\"%2\">%3</a>]\n"
-	    ).arg(firstchar).arg(firstchar).arg(firstchar);
-	    indexLine.append(appendixstr);
-	}
-	os << "<tr><td><a href=\"man:"
-	   << manindex->manpath << "\">\n";
+           const QString appendixstr = QString(
+            " [<a href=\"#%1\" accesskey=\"%2\">%3</a>]\n"
+            ).arg(firstchar).arg(firstchar).arg(firstchar);
+            indexLine.append(appendixstr);
+        }
+        os << "<tr><td><a href=\"man:"
+           << manindex->manpath << "\">\n";
 
-	((char *)manindex->manpage_begin)[manindex->manpage_len] = '\0';
-	os << manindex->manpage_begin
-	   << "</a></td><td>&nbsp;</td><td> "
-	   << (indexmap.contains(manindex->manpage_begin) ? indexmap[manindex->manpage_begin] : "" )
-	   << "</td></tr>"  << endl;
-	last_index = manindex;
+        ((char *)manindex->manpage_begin)[manindex->manpage_len] = '\0';
+        os << manindex->manpage_begin
+           << "</a></td><td>&nbsp;</td><td> "
+           << (indexmap.contains(manindex->manpage_begin) ? indexmap[manindex->manpage_begin] : "" )
+           << "</td></tr>"  << endl;
+        last_index = manindex;
     }
     indexLine.append("</div>");
 
     for (int i=0; i<listlen; i++) {
-	::free(indexlist[i]->manpath);   // allocated by strdup
-	delete indexlist[i];
+        ::free(indexlist[i]->manpath);   // allocated by strdup
+        delete indexlist[i];
     }
 
     delete [] indexlist;
@@ -1367,7 +1360,7 @@ void MANProtocol::listDir(const QUrl &url)
     // But we cannot list man:ls as a directory, this makes no sense (#154173)
 
     if (!title.isEmpty() && title != "/") {
-	error(KIO::ERR_IS_FILE, url.url());
+        error(KIO::ERR_IS_FILE, url.url());
         return;
     }
 
@@ -1378,9 +1371,15 @@ void MANProtocol::listDir(const QUrl &url)
             UDSEntry     uds_entry;
 
             QString name = "man:/(" + *it + ')';
+#if (KIO_VERSION >= QT_VERSION_CHECK(5, 48, 0))
+            uds_entry.fastInsert( KIO::UDSEntry::UDS_NAME, sectionName( *it ) );
+            uds_entry.fastInsert( KIO::UDSEntry::UDS_URL, name );
+            uds_entry.fastInsert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
+#else
             uds_entry.insert( KIO::UDSEntry::UDS_NAME, sectionName( *it ) );
             uds_entry.insert( KIO::UDSEntry::UDS_URL, name );
             uds_entry.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
+#endif
 
             uds_entry_list.append( uds_entry );
         }
@@ -1395,9 +1394,15 @@ void MANProtocol::listDir(const QUrl &url)
         stripExtension( &(*it) );
 
         UDSEntry     uds_entry;
-        uds_entry.insert( KIO::UDSEntry::UDS_NAME, *it );
+#if (KIO_VERSION >= QT_VERSION_CHECK(5, 48, 0))
+        uds_entry.fastInsert(KIO::UDSEntry::UDS_NAME, *it);
+        uds_entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
+        uds_entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("text/html"));
+#else
+        uds_entry.insert(KIO::UDSEntry::UDS_NAME, *it);
         uds_entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
         uds_entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QString::fromLatin1("text/html"));
+#endif
         uds_entry_list.append( uds_entry );
     }
 
